@@ -1,4 +1,4 @@
-/* $Id: hparser.c,v 2.110 2004/11/23 10:56:33 gisle Exp $
+/* $Id: hparser.c,v 2.113 2004/11/23 22:08:54 gisle Exp $
  *
  * Copyright 1999-2004, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
@@ -445,8 +445,13 @@ report_event(PSTATE* p_state,
 			attrval = newSVpvn(beg, len);
 			if (utf8)
 			    SvUTF8_on(attrval);
-			if (!p_state->attr_encoded)
-			    decode_entities(aTHX_ attrval, p_state->entity2char);
+			if (!p_state->attr_encoded) {
+			    if (p_state->utf8_mode)
+				sv_utf8_decode(attrval);
+			    decode_entities(aTHX_ attrval, p_state->entity2char, 0);
+			    if (p_state->utf8_mode)
+				SvUTF8_off(attrval);
+			}
 		    }
 		    else { /* boolean */
 			if (p_state->bool_attr_val)
@@ -510,8 +515,13 @@ report_event(PSTATE* p_state,
 		arg = sv_2mortal(newSVpvn(beg, end - beg));
 		if (utf8)
 		    SvUTF8_on(arg);
-		if (!p_state->is_cdata)
-		    decode_entities(aTHX_ arg, p_state->entity2char);
+		if (!p_state->is_cdata) {
+		    if (p_state->utf8_mode)
+			sv_utf8_decode(arg);
+		    decode_entities(aTHX_ arg, p_state->entity2char, 1);
+		    if (p_state->utf8_mode)
+			SvUTF8_off(arg);
+		}
 	    }
 	    break;
       
@@ -1747,6 +1757,11 @@ parse(pTHX_
 	return;
     }
 
+#ifdef UNICODE_HTML_PARSER
+    if (p_state->utf8_mode)
+	sv_utf8_downgrade(chunk, 0);
+#endif
+
     if (p_state->buf && SvOK(p_state->buf)) {
 	sv_catsv(p_state->buf, chunk);
 	beg = SvPV(p_state->buf, len);
@@ -1765,7 +1780,7 @@ parse(pTHX_
 		(utf8 && len >= 6 && strnEQ(beg, "\xC3\xAF\xC2\xBB\xC2\xBF", 6))
 	       )
 	    {
-		if (p_state->argspec_entity_decode)
+		if (p_state->argspec_entity_decode && !p_state->utf8_mode)
 		    warn("Parsing of undecoded UTF-8 will give garbage when decoding entities");
 	    }
 	    if (DOWARN && utf8 && len >= 2 && strnEQ(beg, "\xFF\xFE", 2)) {
